#!/usr/bin/env python3

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""Version and general info
"""

PROGRAM_NAME: str = 'libmonty'

__version__: str = '0.1.1'
